/*********************************************************************************

Copyright(c) 2015 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/
/*
 * Project		SHARC L1-L1 using MDMA example - Dual SHARC Talk through.
 * Description  Four channels in, SHARC0 filters two channels while SHARC1 filters
 *              the other two channels in parallel, resulting in four filtered channels.
 * 	            The purpose of this project is to demonstrate using
 * 				two MDMA channels to move raw and filtered data from SHARC0 to SHARC1.
 * 				The project takes four channels of audio from the ADAU1979 CODEC into
 * 				SHARC0 L1 SRAM and sends the raw data to SHARC1 L1 SRAM using MDAM channel A0.
 * 				SHARC0 processes two of the four channels at the time that SHARC1
 * 				is processing the other two channels (parallel processing).
 * 				After SHARC0 completes its processing it uses MDMA channel A1 to
 * 				move the filtered audio channels to SHARC1 L1 SRAM. SHARC1 combines its filtered audio channels
 * 				with the filtered audio channels from SHARC0 resulting in four channels of filtered audio.
 * 				The resulting four channels of filtered audio is sent from SHARC1 L1 SRAM to
 * 				the ADAU1962 DAC driver running on SHARC1. The ARM is used during initialization
 * 				but is not used during runtime.
 *
 *              SHARC0							| SHARC1
 *              ---------------------------------------------------------------
 *			T0	Init ADAU1979 ADC				| Init ADAU1962 DAC
 *			T1	Initialize MDMA channels		| Wait for MDMA init complete
 *			T2	Wait for MDMA interrupt install | Install MDMA interrupt handlers
 *			T3  Enable ADAU1979					| Wait for first audio frame
 *
 *			T4	Send raw audio frame over MDMAA0| MDMA A0 complete callback
 *			T5	Filter audio channels 0,2		| Filter audio channels 1,3  <<<<<<<<<<<<<<<<<<<
 *			T6	Send filtered audio over MDMA A1| MDMA A1 complete callback
 *			T7	Wait for another audio frame	| Merge audio channels, send to ADAU1962 DAC
 *
 * 				--------------   -------------------------------------
 * 				|            |   |     SHARC0 filters channels 0,2   |
 * 				|  ADAU1979  |-->| 4 channels in                     |
 * 				|            |   |                     filtered audio|
 * 				--------------   -------------------------------------
 * 				                      | MDMA A0             | MDMA A1
 * 				                      | raw audio           | 2 channels filtered
 * 				                 ----------------------------------------   ------------
 * 				                 |SHARC1 filters channels 1,3 and merges|   |          |
 *                               |                        4 channels out|-->| ADAU1962 |
 *                               |  SHARC0 chan 0,2 + SHARC1 chan 1,3   |   |          |
 *                               ----------------------------------------   ------------
 *
 *				Griffin's MDMA hardware provides the unique ability to perform data transfer and
 *				signaling in the same package. Data can be transfered directly from SHARC0 L1 to
 *				SHARC1 L1 with both SHARC's getting signaled when the transfer completes. Multiple
 *				MDMA channels are supported and can run concurrently. Each channel provides a unique
 *				signal when the transfer completes.
 */
#include <sys/platform.h>
#include <sys/adi_core.h>
#include <stdio.h>
#include "adi_initialize.h"
#include <drivers/dac/adau1962a/adi_adau1962a.h>
#include "MultiCoreTalkThruSharcSharc_Core2.h"
#include "SHARC_linkInterface.h"


#define TIMESTAMP	if(CycleCountBufferIndex < 1024) {CycleCountBuffer[CycleCountBufferIndex][0] =__LINE__; CycleCountBuffer[CycleCountBufferIndex++][1] = __builtin_emuclk();}

//*****************************************************************************
// External Functions
//*****************************************************************************
extern uint32_t Adau1962aSubmitBuffers(void);
extern uint32_t Adau1962aInit(void);
extern uint32_t Adau1962DoneWithBuffer( void *pBuffer );
extern uint32_t Adau1962Enable( void );
extern void SHARC1Filter( int8_t *InputBuffer, int8_t *OutputBuffer, uint32_t BytesToProcess );


//*****************************************************************************
// Local Variables
//*****************************************************************************
uint32_t DacCount = 0u;
uint32_t RAWBuffersReceived = 0u;
uint32_t FILTEREDBuffersReceived = 0u;
uint8_t  DacStarted = 0u;
void *pSportOutputBuffer = 0;
uint32_t DMASlaveDestinationAddress = 0u;
int8_t FilteredData[AUDIO_BUFFER_SIZE];

// These variables are used for instrumenting the code
volatile int CycleCountBufferIndex = 0;
volatile unsigned long CycleCountBuffer[4096][2];

//*****************************************************************************
// Merge audio channels
//
// Audio Format (32 bits/sample) - interleaved (CH0, CH1, CH2, Ch3, ...)
//*****************************************************************************
void MergeAudioChannels( void *InputBuffer1, void *InputBuffer2, void *OutputBuffer, uint32_t BytesToProcess )
{
	int32_t *pSrc1 = InputBuffer1;
	int32_t *pSrc2 = InputBuffer2;
	int32_t *pDst  = OutputBuffer;

	pSrc2++;

	// InputBuffer1 channels 0,2
	// InputBuffer2 channels 1,3
	for(uint32_t i=0; i<BytesToProcess/4;i+=4)
	{
		*pDst++ = *pSrc1; // Channel 0
		pSrc1 += 2;
		*pDst++ = *pSrc2; // Channel 1
		pSrc2 += 2;
		*pDst++ = *pSrc1; // Channel 2
		pSrc1 += 2;
		*pDst++ = *pSrc2; // channel 3
		pSrc2 += 2;
	}
}

//*************************************************************************
// Interrupt handler for MDMA RAW data transfer complete
//*************************************************************************
static void RawDataTransferFromMasterComplete(uint32_t SID, void *pCBParam)
{
	static uint32_t PreviousFrameCounter = 0xFFFFu;
	uint32_t CounterFromMasterSHARC = *sharc_flag_in_L2;

	if( CounterFromMasterSHARC != PreviousFrameCounter )	// Fix for stutter
	{
		PreviousFrameCounter = CounterFromMasterSHARC;
		TIMESTAMP
		SHARC1Filter( (int8_t *)MDMA_LOCAL_ADDR, FilteredData, AUDIO_BUFFER_SIZE );
		++RAWBuffersReceived;
	}
}

//*************************************************************************
// Interrupt handler for MDMA FILTERED data transfer complete
//*************************************************************************
static void FilteredDataTransferFromMasterComplete(uint32_t SID, void *pCBParam)
{
	static uint32_t PreviousFrameCounter = 0xFFFFu;
	uint32_t CounterFromMasterSHARC = *sharc_flag_in_L2;

	if( CounterFromMasterSHARC != PreviousFrameCounter )	// Fix for stutter
	{
		PreviousFrameCounter = CounterFromMasterSHARC;
		TIMESTAMP
		++FILTEREDBuffersReceived;

		// Merge the FilteredData buffer with the audio frame just received from the Master SHARC.
		if( pSportOutputBuffer != 0 )
			MergeAudioChannels( (void *)(MDMA_LOCAL_ADDR+AUDIO_BUFFER_SIZE), FilteredData, pSportOutputBuffer, AUDIO_BUFFER_SIZE );
	}
}

/** 
 * If you want to use command program arguments, then place them in the following string. 
 */
char __argv_string[] = "";

//*****************************************************************************
// SHARC1 core 2
//
// SHARC1 takes in four raw audio channels from SHARC0 via MDMA A0.
// It filters 2/4 channels and merges the channels with the audio
// frame received via MDMA A1. The resulting four filtered channels are
// output via the ADAU1962 DAC.
//*****************************************************************************
int main(int argc, char *argv[])
{
	uint32_t 	Result = 0u;

	//*************************************************************************
	// Initialize managed drivers and/or services that have been added to
	// the project.
	// @return zero on success
	//*************************************************************************
	adi_initComponents();
	
	//*************************************************************************
	// Initialize ADAU1962A
	//*************************************************************************
	if(Adau1962aInit() != 0u)
	{
		fprintf(stdout, "Core2: Adau1962aInit() failed\n" );
		while(1){}
	}

	if(Adau1962aSubmitBuffers() != 0u)
	{
		fprintf(stdout, "Core2: Adau1962aSubmitBuffers() failed\n" );
		while(1){;}
	}

	//*************************************************************************
	// Initialize MDMA - Code blocks until both cores complete init
	//*************************************************************************
	Result = SHARC_linkSlaveInit( RawDataTransferFromMasterComplete, FilteredDataTransferFromMasterComplete, (void *)DMASlaveDestinationAddress );
	if(Result != 0u)
	{
		fprintf(stdout, "Core2: SHARClinkSlaveInit() failed\n" );
		while(1){;}
	}

	//*************************************************************************
	// For synchronization, wait until we receive the first MDMA A0 (raw)
	// audio frame before enabling the ADAU1962 driver.
	// After the driver is enabled, all processing is done in the MDMA callbacks.
	//*************************************************************************
	while(1)
	{
		if( DacStarted == 0 && RAWBuffersReceived > 0 )
		{
			// Enable audio output
			Adau1962Enable();
			DacStarted = 1;
			fprintf(stdout, "Core2: DAC Started\n" );
		}
	}

	return 0;
}


//*****************************************************************************
// DAC callback - Called when the DAC requires data
//				  The MDMA A1 callback MUST occur before this callback.
//				  The user must make sure the filters do not take more time
//				  than is available in the frame.
//*****************************************************************************
void DacCallback(void *pCBParam, uint32_t nEvent, void *pArg)
{

    switch(nEvent)
    {
        case ADI_SPORT_EVENT_TX_BUFFER_PROCESSED:
        	/* Update callback count */
        	DacCount++;
        	TIMESTAMP
        	// We can still copy to the buffer after it is submitted to the driver
        	pSportOutputBuffer = pArg;
    		Adau1962DoneWithBuffer( pArg );
        	break;
        default:
            break;
    }
}


